@@ -13,6 +13,7 @@ The following extensions are currently implemented:
     * ability to set (advisory) thread priority
     * ability to set (advisory) CPU affinity at thread and process level
     * thread groups for simultaneous management of multiple threads
+    * SHLock class for shared/exclusive (also known as read/write) locks
 
 The following API niceties are also included:
 
@@ -39,8 +40,8 @@ from __future__ import with_statement
 
 
 __ver_major__ = 0
-__ver_minor__ = 1
-__ver_patch__ = 4
+__ver_minor__ = 2
+__ver_patch__ = 0
 __ver_sub__ = ""
 __version__ = "%d.%d.%d%s" % (__ver_major__,__ver_minor__,
                               __ver_patch__,__ver_sub__)
@@ -65,7 +66,7 @@ except ImportError:
 __all__ = ["active_count","activeCount","Condition","current_thread",
            "currentThread","enumerate","Event","local","Lock","RLock",
            "Semaphore","BoundedSemaphore","Thread","ThreadGroup","Timer",
-           "setprofile","settrace","stack_size","group_local",
+           "SHLock","setprofile","settrace","stack_size","group_local",
            "CPUSet","system_affinity","process_affinity"]
 
 
